#include "sprites.h"
#include "transform.h"

FONT* Font;

//This tells me the 'resolution' of my game (in this case 320x240) and the size of my window.
int screenx = 320;
int screeny = 240;
int windowx = 960;
int windowy = 720;

//This is a quick and dirty way to track the Transform I'm using.
typedef enum TransformTypes { RESIZE, RUMBLE, SHOWER, WAVY, TWISTY, ROTATE, STICKY, BLACKHOLE } TT;
TT currentTransform = RESIZE;

//This is a simple timer function.
volatile int Time = 0;
void TimerFunc()
{
    Time++;
}
END_OF_FUNCTION(TimerFunc);



//When the black hole is active, we draw the hole in a lighter background.
void DrawBackground(float x, float y, float radius1, float radius2)
{
    glBegin(GL_QUADS);
    glColor4ub(100,100,100, 255);
    glVertex2f(0, 0);
    glVertex2f(windowx, 0);
    glVertex2f(windowx, windowy);
    glVertex2f(0, windowy);
    glEnd();

    glBegin(GL_TRIANGLE_FAN);
    glColor4ub(50,50,50,255);
    glVertex2f(x, y);
    //This creates a triangle fan in 360 steps, stepping around a radius.  Obviously we get a circle.
    for (int angle = 0; angle < 360; angle++)
    {
        glVertex2f(x + sin(angle) * radius1, y + cos(angle) * radius1);
    }
    glEnd();

    glBegin(GL_TRIANGLE_FAN);
    glColor4ub(0,0,0,255);
    glVertex2f(x, y);
    for (int angle = 0; angle < 360; angle++)
    {
        glVertex2f(x + sin(angle) * radius2, y + cos(angle) * radius2);
    }
    glEnd();
}

//This draws the numbers so you know what transform you're on.
void DrawNumbers()
{
 glPushMatrix();
 glTranslatef(140,219,0);
 for(int i = 0; i < 8; i++)
 {
    glColor4ub(255,255,255, 255);
    glTranslatef(11,0,0);
    glBegin(GL_QUADS);
    //This draws the white border.
    glVertex2f(0, 0);
    glVertex2f(10, 0);
    glVertex2f(10, 11);
    glVertex2f(0, 11);
    glEnd();
    //This is the inner color.
    glBegin(GL_QUADS);
    if(i == currentTransform) glColor4ub(255,0,0, 255);
    else glColor4ub(80,0,0, 255);
    glVertex2f(1, 1);
    glVertex2f(9, 1);
    glVertex2f(9, 10);
    glVertex2f(1, 10);
    glEnd();
    glScalef(1,-1,1);
    allegro_gl_printf(Font, 3, -8, 0, makecol(255, 255, 255), "%d", i+1);
    glScalef(1,-1,1);
 }
 glPopMatrix();
}

//This takes each pixel and passes it to my draw function.
void DrawPixels(Pixels& theImage)
{
    glLoadIdentity();
    Pixel* startPixel = theImage.GetFirstPixel();

    while (startPixel != 0)
    {
        DrawPixel(startPixel, theImage.X(), theImage.Y());
        startPixel = startPixel->next;
    }
}

//This part is crucial for starting up.
void Initialize()
{
    /* Initialize both Allegro and AllegroGL */
    allegro_init();
    install_allegro_gl();

    /* Tell Allegro we want to use the keyboard, mouse, and timer. */
    install_keyboard();
    install_mouse();
    install_timer();
    show_os_cursor(MOUSE_CURSOR_ARROW);  //I'll be using the standard mouse cursor for this game.
    allegro_gl_set(AGL_Z_DEPTH, 8);
    allegro_gl_set(AGL_COLOR_DEPTH, 32);
    allegro_gl_set(AGL_SUGGEST, AGL_Z_DEPTH | AGL_COLOR_DEPTH);  //32-bit color means alpha compatibility.
    allegro_gl_set(AGL_REQUIRE, AGL_DOUBLEBUFFER);  //Double buffering is good.
    set_window_title("PDD - Prototyping");

    set_gfx_mode(GFX_OPENGL_WINDOWED, windowx, windowy, 0, 0);

    //This tells OpenGL that I'm using a flat surface with no z value to account for.
    //An orthographic projection with our given resolution gives us a 2D game.
    glMatrixMode(GL_PROJECTION);
    glOrtho(0, screenx,screeny, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glDisable(GL_DEPTH_TEST);  // We don't use z.
    glEnable(GL_BLEND);  //We do use alpha blending.
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    Font = allegro_gl_load_system_font_ex("Arial", AGL_FONT_TYPE_BITMAP, AGL_FONT_STYLE_BOLD, 10, 20, 1.0, 0, 256);
}

//This is where all the main stuff happens.  I wrote it in a hurry to demonstrate the rest of the code.  It's kind of a mess.
int PlayLoop()
{
    //We load up our testing image (in this case called ship... this is because I was testing it with ships earlier.)
    Pixels Ship = LoadPixels("test.png");
    LOCK_VARIABLE(Time);
    LOCK_FUNCTION(TimerFunc);
    install_int_ex(TimerFunc, BPS_TO_TIMER(60));
    //Now we have our timer set to 60FPS.

 //Below are all the variables we'll use to handle user controls.  MousePressed is set when the left mouse button is down.
 //If Pickup is set, it means we just hit our target image.  SetKeyH is just for handling 'h' without key repeating.
    bool MousePressed = false;
    bool Pickup = false;
    bool SetKeyH = false;

    //Here are all the transforms we'll be using today.
    RumbleTransform* theRumble = new RumbleTransform();
    StickyTransform* theSticky = new StickyTransform();
    ShowerTransform* theShower = new ShowerTransform();
    TwistyTransform* theTwisty = new TwistyTransform();
    RotateTransform* theRotate = new RotateTransform();
    SizeTransform* theSize = new SizeTransform();
    BlackHoleTransform* theHole = new BlackHoleTransform(100,40,20,60);
    theHole->SetTransform(&Ship); //Since the black hole is *always* on, we set it up now.
    WavyTransform* theWavy = new WavyTransform();
    float wavyAmplitude = 3.0;
    float wavyFrequency = 0.1;
    float wavySpeed = 0.3;
    Transform* activeTransform = theSize;


    Ship.SetX(100);
    Ship.SetY(20);
    while (!key[KEY_ESC]) //We quit on escape.
    {
        if (Time > 0) //If the timer goes off, render another frame.
        {
            poll_mouse();
            // We have to translate mouse-window co-ords into mouse-projection co-ords.
            float pix_coordx = mouse_x * (screenx/(windowx*1.0));
            float pix_coordy = mouse_y * (screeny/(windowy*1.0));

            //We move the ship with wasd movement.
            if (key[KEY_A]) Ship.SetX(Ship.X()-2);
            else if (key[KEY_D]) Ship.SetX(Ship.X()+2);
            if (key[KEY_S]) Ship.SetY(Ship.Y()+2);
            else if (key[KEY_W]) Ship.SetY(Ship.Y()-2);

            //If we have a chance to switch out transforms (always an option with a black hole)...
            if (!activeTransform->IsTransformOn() || currentTransform == BLACKHOLE)
            {
                if (key[KEY_1] && currentTransform != RESIZE)
                {
                    activeTransform = theSize;
                    currentTransform = RESIZE;
                }
                else if (key[KEY_2] && currentTransform != RUMBLE)
                {
                    activeTransform = theRumble;
                    currentTransform = RUMBLE;
                }
                else if (key[KEY_3] && currentTransform != SHOWER)
                {
                    activeTransform = theShower;
                    currentTransform = SHOWER;
                }
                else if (key[KEY_4] && currentTransform != WAVY)
                {
                    activeTransform = theWavy;
                    currentTransform = WAVY;
                }
                else if (key[KEY_5] && currentTransform != TWISTY)
                {
                    activeTransform = theTwisty;
                    currentTransform = TWISTY;
                }
                else if (key[KEY_6] && currentTransform != ROTATE)
                {
                    activeTransform = theRotate;
                    currentTransform = ROTATE;
                }
                else if (key[KEY_7] && currentTransform != STICKY)
                {
                    activeTransform = theSticky;
                    currentTransform = STICKY;
                }
                else if (key[KEY_8] && currentTransform != BLACKHOLE)
                {
                    activeTransform = theHole;
                    theHole->SwitchTransform();
                    currentTransform = BLACKHOLE;
                }
                //We turn off our black hole when we aren't using it.
                if (currentTransform != BLACKHOLE && theHole->IsTransformOn()) theHole->SwitchTransform();
            }

            /*
            Throughout the rest of this function, you'll see dynamic casting.  Our Active Transform is a generic Transform
            pointer that can point to any type of Transform.  When it points at a particular Transform, we must cast it to that class.
            */
            //The following keys let us change wavy and resize.  It will stretch and shrink the images following the arrows accordingly.
            if (key[KEY_LEFT] && currentTransform == WAVY)
            {
                wavyAmplitude -= 0.2;
                dynamic_cast<WavyTransform*> (activeTransform)->SetAmplitude(wavyAmplitude);
            }
            else if (key[KEY_LEFT] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->DownHSize();

            if (key[KEY_RIGHT] && currentTransform == WAVY)
            {
                wavyAmplitude += 0.2;
                dynamic_cast<WavyTransform*> (activeTransform)->SetAmplitude(wavyAmplitude);
            }
            else if (key[KEY_RIGHT] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->UpHSize();

            if (key[KEY_UP] && currentTransform == WAVY)
            {
                wavyFrequency += 0.01;
                dynamic_cast<WavyTransform*> (activeTransform)->SetFrequency(wavyFrequency);
            }
            else if (key[KEY_UP] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->UpVSize();

            if (key[KEY_DOWN] && currentTransform == WAVY)
            {
                wavyFrequency -= 0.01;
                dynamic_cast<WavyTransform*> (activeTransform)->SetFrequency(wavyFrequency);
            }
            else if (key[KEY_DOWN] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->DownVSize();

            if (key[KEY_PLUS_PAD] && currentTransform == WAVY)
            {
                wavySpeed += 0.05;
                dynamic_cast<WavyTransform*> (activeTransform)->SetSpeed(wavySpeed);
            }
            else if (key[KEY_PLUS_PAD] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->UpSize();

            if (key[KEY_MINUS_PAD] && currentTransform == WAVY)
            {
                wavySpeed -= 0.05;
                dynamic_cast<WavyTransform*> (activeTransform)->SetSpeed(wavySpeed);
            }
            else if (key[KEY_MINUS_PAD] && currentTransform == RESIZE)
                dynamic_cast<SizeTransform*> (activeTransform)->DownSize();

                //This is if H is pressed.  Once pressed, we stop registering it as a hit.  This prevents stuttering and repeating.
            if (key[KEY_H]) SetKeyH = true;
            if (!key[KEY_H] && SetKeyH && currentTransform == WAVY)
            {
                dynamic_cast<WavyTransform*> (activeTransform)->SetHorizontal();
                SetKeyH = false;
            }
            else if (!key[KEY_H] && SetKeyH && currentTransform == TWISTY)
            {
                dynamic_cast<TwistyTransform*> (activeTransform)->SetHorizontal();
                SetKeyH = false;
            }

            if (mouse_b & 1 && !MousePressed && !activeTransform->IsTransformOn())
            {
                MousePressed = true;
                //If we score a hit with our button press, we set up the transform with the target image.
                if (Ship.CheckHit(pix_coordx, pix_coordy))
                {
                    if (currentTransform == STICKY) dynamic_cast<StickyTransform*> (activeTransform)->SetTransform(&Ship, pix_coordx, pix_coordy, Ship.GetLastHit());
                    else if (currentTransform == RUMBLE) dynamic_cast<RumbleTransform*> (activeTransform)->SetTransform(&Ship);
                    else if (currentTransform == WAVY) dynamic_cast<WavyTransform*> (activeTransform)->SetTransform(&Ship, 0, 0, screenx, screeny);
                    else if (currentTransform == SHOWER) dynamic_cast<ShowerTransform*> (activeTransform)->SetTransform(&Ship, 0, 0, screenx, screeny);
                    else if (currentTransform == TWISTY) dynamic_cast<TwistyTransform*> (activeTransform)->SetTransform(&Ship);
                    else if (currentTransform == ROTATE) dynamic_cast<RotateTransform*> (activeTransform)->SetTransform(&Ship);
                    else if (currentTransform == RESIZE) dynamic_cast<SizeTransform*> (activeTransform)->SetTransform(&Ship);
                    //Then we turn it on.
                    if (!activeTransform->IsTransformOn())
                        activeTransform->SwitchTransform();
                }
            }
            //If they let go of the mouse button...
            if (!(mouse_b&1)) MousePressed = false;

            //If they let go and the transform is on, we turn it off.  The black hole is an exception as it is always on.
            if (MousePressed == false && activeTransform->IsTransformOn() && currentTransform != BLACKHOLE)
            {
                activeTransform->SwitchTransform();
            }

            //If it's on, we update the transform with any relevant information.
            if (activeTransform->IsTransformOn())
            {
                if (currentTransform == STICKY) dynamic_cast<StickyTransform*> (activeTransform)->UpdateTransform(pix_coordx-0.25, pix_coordy-0.25);
                else if (currentTransform == RUMBLE) dynamic_cast<RumbleTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == WAVY) dynamic_cast<WavyTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == SHOWER) dynamic_cast<ShowerTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == TWISTY) dynamic_cast<TwistyTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == ROTATE) dynamic_cast<RotateTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == RESIZE)  dynamic_cast<SizeTransform*> (activeTransform)->UpdateTransform();
                else if (currentTransform == BLACKHOLE)  dynamic_cast<BlackHoleTransform*> (activeTransform)->UpdateTransform();
            }
            //Draw the image.  We draw the background first if the black hole is set.
            glClear(GL_COLOR_BUFFER_BIT);
            if (currentTransform == BLACKHOLE)
                DrawBackground(theHole->GetX(), theHole->GetY(), theHole->GetEventHorizon(), theHole->GetBlackHole());
            DrawPixels(Ship);

            //This next part draws the text information as it relates to the active transform.
            //It just gives you some instructions if necessary.  You'll notice I print each message twice.
            //The second printing is for the shadow.
            glPushMatrix();
            glScalef(1, -1, 1);
            glTranslatef(10, 0.95*(-screeny), 0);
            if (currentTransform == RUMBLE)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "RUMBLE MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "RUMBLE MODE SET");
            }
            else if (currentTransform == STICKY)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "STICKY MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "STICKY MODE SET");
            }
            else if (currentTransform == WAVY)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "WAVY MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "WAVY MODE SET");
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "-  Speed %4.2f  + ", wavySpeed);
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "-  Speed %4.2f  + ", wavySpeed);
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "<- Amplitude %4.2f -> ", wavyAmplitude);
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "<- Amplitude %4.2f -> ", wavyAmplitude);
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "^  Frequency %4.2f  v ", wavyFrequency);
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "^  Frequency %4.2f  v ", wavyFrequency);
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "H: Switch Horizontal/Vertical ");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "H: Switch Horizontal/Vertical ");
            }
            else if (currentTransform == SHOWER)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "SHOWER MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "SHOWER MODE SET");
            }
            else if (currentTransform == RESIZE)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "SIZE MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "SIZE MODE SET");
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "<-  H Size  ->");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "<-  H Size  ->");
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "^  V Size  v");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "^  V Size  v");
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "-  Size  +");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "-  Size  +");
            }
            else if (currentTransform == ROTATE)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "ROTATE MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "ROTATE MODE SET");
            }
            else if (currentTransform == BLACKHOLE)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "BLACK HOLE MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "BLACK HOLE MODE SET");
            }
            else if (currentTransform == TWISTY)
            {
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "TWISTY MODE SET");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "TWISTY MODE SET");
                glTranslatef(0, 0.04*screeny, 0);
                allegro_gl_printf(Font, 0, 0, 0, makecol(100, 100, 100), "H: Switch Horizontal/Vertical ");
                allegro_gl_printf(Font, screenx/320, screeny/240, 0, makecol(255, 255, 255), "H: Switch Horizontal/Vertical ");
            }
            glPopMatrix();
            DrawNumbers();  //We draw our numbers so we know which transform is active.
            allegro_gl_flip();  //We then flip it to the front and show it to the world.
            Time = 0;           //And reset our timer.
        }
    }

    //Once we're done, we delete anything dynamically allocated.
    Ship.~Pixels();  //This probably looks a bit odd.  It's because I know the pixels are allocated, but Ship isn't a pointer.
    delete theRumble;
    delete theSticky;
    delete theSize;
    delete theShower;
    delete theTwisty;
    delete theRotate;
    delete theWavy;
    delete theHole;
}

int main()
{
    Initialize();
    PlayLoop();

    /* Finished. */
    return 0;
}
END_OF_MAIN();
