#include "pixels.h"
#include <stdio.h>

//This default constructor creates a blank 8x8 black image.
Pixels::Pixels()
{
    PixelHead = 0;
    PixelCurrent = 0;
    PixelTail = 0;
    LastHit = 0;
    originX = 0;
    originY = 0;
}

//This destructor properly removes the declared memory.
Pixels::~Pixels()
{
    PixelCurrent = PixelHead;
    while (PixelCurrent != 0)
    {
        PixelHead = PixelHead->next;
        delete PixelCurrent;
        PixelCurrent = PixelHead;
    }
}

//Returns the furthest endpoint in the pixels collection.
float Pixels::EndX()
{
    return endX;
}

//Returns the furthest endpoint in the pixels collection.
float Pixels::EndY()
{
    return endY;
}

//Returns the top-left point in the pixels collection.
float Pixels::X()
{
    return originX;
}

//Returns the top-left point in the pixels collection.
float Pixels::Y()
{
    return originY;
}

//Moves the pixels to a new spot.
void Pixels::SetX(float nx)
{
    originX = nx;
}

//Moves the pixels to a new spot.
void Pixels::SetY(float ny)
{
    originY = ny;
}

//This resets our ability to traverse the list.
Pixel* Pixels::GetFirstPixel()
{
    PixelCurrent = PixelHead;
    return PixelHead;
}

//This gets the last pixel in the list.
Pixel* Pixels::GetLastPixel()
{
    return PixelTail;
}

//This gets the next pixel in the list.
Pixel* Pixels::GetNextPixel()
{
    PixelCurrent = PixelCurrent->next;
    return PixelCurrent;
}

//This moves the end-point to a new set of co-ordinates.  It does not need to match an existing pixel and is mainly used
//by Transforms to calculate the motion of nearby pixels.
void Pixels::MoveEnd(float nx, float ny)
{
    endX = nx;
    endY = ny;
}

//This adds a pixel into the fray.
void Pixels::AddPixel(float rx, float ry, int r, int g, int b, int a, int hit)
{
    Pixel* NewPixel = new Pixel;

    NewPixel->r = r;
    NewPixel->g = g;
    NewPixel->b = b;
    NewPixel->a = a;
    NewPixel->oa = a;
    NewPixel->hit = hit;
    NewPixel->rx = rx;
    NewPixel->ry = ry;
    NewPixel->ox = rx;
    NewPixel->oy = ry;
    NewPixel->ax = 0;
    NewPixel->ay = 0;
    NewPixel->vx = 0;
    NewPixel->vy = 0;
    NewPixel->next = 0;
    NewPixel->prev = 0;
    NewPixel->z = 0;

//Now add this new pixel to the list.
    if (PixelHead == 0)  //If we're starting our list, set it up.
    {
        PixelHead = NewPixel;
        PixelTail = NewPixel;
    }
    else  //Otherwise, add it to the end of the list.
    {
        PixelTail->next = NewPixel;
        NewPixel->prev = PixelTail;
        PixelTail = NewPixel;
    }
    //If this endpoint is further from the origin, we set that up too.
    if (NewPixel->rx >= endX && NewPixel->ry >= endY)
    {
        endX = NewPixel->rx;
        endY = NewPixel->ry;
    }
}

//This lets me know if the image does have a pixel at the corresponding co-ordinate (give or take a unit.)
bool Pixels::CheckHit(float coord_x, float coord_y)
{
    coord_x -= originX + 1;  //We center those co-ordinates according to our origin.
    coord_y -= originY + 1;

//Now we check the list of pixels for a match.

    PixelCurrent = PixelHead;
    while (PixelCurrent != 0)
    {
        if (PixelCurrent->rx >= coord_x && PixelCurrent->rx <= coord_x+1 &&
                PixelCurrent->ry >= coord_y && PixelCurrent->ry <= coord_y+1 &&
                PixelCurrent->hit)
        {
            LastHit = PixelCurrent;
            return true;
        }
        PixelCurrent = PixelCurrent->next;
    }

    return false;
}

//If the image was recently 'hit', we find out what pixel was affected by that hit.
Pixel* Pixels::GetLastHit()
{
    return LastHit;
}
