#include "transform.h"

//Basic constructor.
RotateTransform::RotateTransform()
{
    theImage = 0;
    rotation = 0.03;
    centerX = 0;
    centerY = 0;
    Active = false;
    Snap = false;
}

//This function sets up the Transform to affect our current Target.
void RotateTransform::SetTransform(Pixels* Target)
{
    theImage = Target;
    //We need to rotate about a center.  This should be the mid-point relative to the endpoint, right?
    centerX = (theImage->EndX()) / 2.0;
    centerY = (theImage->EndY()) / 2.0;
}

//This function tells the pixels what to do.
void RotateTransform::UpdateTransform()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    //We are going to rotate the image by degrees based on our 'rotation' variable.
    while (thePixel != 0)
    {
        //The rotation happens about the midpoint of our image.  We rotate all pixels on a circle relative to that midpoint.
        float dCos = cos(rotation);
        float dSin = sin(rotation);
        float ox = thePixel->rx - centerX;
        float oy = thePixel->ry - centerY;

        //What is below is the way you calculate movement along a circular path.  You can google to figure out the math.
        thePixel->rx = (ox * dCos) - (oy * dSin) + centerX;
        thePixel->ry = (ox * dSin) + (oy * dCos) + centerY;

        thePixel = theImage->GetNextPixel();
    }
}

void RotateTransform::SwitchTransform()
{
    if (Active)  //When we turn it off, we make the rotation a permanent effect, resetting the origin co-ordinates of each pixel.
    {
        rotation = 0;
        Pixel* thePixel = theImage->GetFirstPixel();
        while (thePixel != 0)
        {
            thePixel->ox = thePixel->rx;
            thePixel->oy = thePixel->ry;

            thePixel = theImage->GetNextPixel();
        }
    }

    Active = !Active;
}

bool RotateTransform::IsTransformOn()
{
    return Active;
}
