#include "transform.h"

//basic constructor.
RumbleTransform::RumbleTransform()
{
    theImage = 0;
    centerX = 0;
    centerY = 0;
    radius = 0;
    Active = false;
    Snap = false;
}

//This sets us up with a target.  We get the mid-point of our image so we can 'rumble away' from that point.
void RumbleTransform::SetTransform(Pixels* Target)
{
    theImage = Target;
    centerX = 0;
    centerY = 0;
    //The midpoint is relative to the endpoint, right?
    centerX = Target->EndX()/2.0;
    centerY = Target->EndY()/2.0;
}

//When the Transform is 'let go', this function tells pixels how to snap back in place.
void RumbleTransform::SnapIt()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    int Continue = 0;
    while (thePixel != 0)
    {
        if (thePixel->rx == thePixel->ox && thePixel->ry == thePixel->oy)
        {
         //If the pixel is sitting upon it's original position, we reset it's alpha value to be visible.
            thePixel->a = thePixel->oa;
        }
        else
        {   //Calculate the vector between the original position and the current position.
            //Then we divide it by 10, making it take 10 steps to return to normal.
            float movex = (thePixel->ox - thePixel->rx) / 10.0;
            float movey = (thePixel->oy - thePixel->ry) / 10.0;

            thePixel->rx += movex;  //Then we move to this new position.
            thePixel->ry += movey;

            //If we're 'close enough', we just set it to the original position.
            if (thePixel->rx > thePixel->ox-0.5 && thePixel->rx < thePixel->ox+0.5) thePixel->rx = thePixel->ox;
            if (thePixel->ry > thePixel->oy-0.5 && thePixel->ry < thePixel->oy+0.5) thePixel->ry = thePixel->oy;

            //We restore some of it's visibility.
            thePixel->a += 4;
            if (thePixel->a > thePixel->oa) thePixel->a = thePixel->oa;
            Continue++;  //If this counts up, we're not done snapping back.
        }
        thePixel = theImage->GetNextPixel();
    }
    if (!Continue)  //If no pixels were moved.
    {
        Active = false;  //We're done and we can now turn off our Transform.
        Snap = false;
    }
}

//This rumbles each pixel in the image away from the image's central point.
void RumbleTransform::RumbleIt()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    while (thePixel != 0)
    {
        //We figure out the vector length between our pixel's current position and the mid-point. 
        //Then we take a piece of it and move along that length.
        float movex = (thePixel->rx - centerX) / 60.0;
        float movey = (thePixel->ry - centerY) / 60.0;

        //We also want to add some random motion for a rumbling effect.
        float randomx = ((rand() % 10) - 4.7)/ 7.0;
        float randomy = ((rand() % 10) - 4.7)/ 7.0;
        thePixel->rx += movex + randomx;
        thePixel->ry += movey + randomy;

        //The distance below is important.  The further away we get from the original position, the harder our alpha value gets hit.
        float distance = (thePixel->rx-thePixel->ox)*(thePixel->rx-thePixel->ox) + (thePixel->ry-thePixel->oy)*(thePixel->ry-thePixel->oy);
        thePixel->a -= (int) distance/200;
        if (thePixel->a < 0) thePixel->a = 0;

        thePixel = theImage->GetNextPixel();
    }
}

//We snap if we're letting go, else we rumble.
void RumbleTransform::UpdateTransform()
{
    if (Snap)
    {
        SnapIt();
    }
    else
    {
        RumbleIt();
    }
}

void RumbleTransform::SwitchTransform()
{
    if (Active)  //We now get ready to turn ourselves off.  We set the snapping motion on.
    {            //Once the snapping motion is done, the Transform will turn itself off.
        Snap = true;
    }
    else Active = true;
}

bool RumbleTransform::IsTransformOn()
{
    return Active;
}
