#include "transform.h"

//basic constructor.
ShowerTransform::ShowerTransform()
{
    theImage = 0;
    Active = false;
}

//This sets the image we're targeting.  We also get the box co-ordinates, letting us know the edges of the affected area.
//The pixels will bounce off these lines, effectively keeping them inside a given area.
void ShowerTransform::SetTransform(Pixels* Target, float sx, float sy, float ex, float ey)
{
    startX = sx;
    startY = sy;
    endX = ex;
    endY = ey;
    theImage = Target;
    Pixel* thePixel = theImage->GetFirstPixel();
    while (thePixel != 0)
    {
        float randomx = ((rand() % 20) - 9.7)/ 7.0;
        float randomy = ((rand() % 10) - 10.7)/ 10.0;
        thePixel->vx += randomx;  //This gives each pixel a random velocity, upon which it will act.
        thePixel->vy += randomy;
        thePixel->ay += 0.05;  //There is also gravity to consider.  This acceleration will constantly pull downwards on the velocity.

        thePixel = theImage->GetNextPixel();
    }
}

//This transform will take each pixel and move it according to it's velocity and acceleration settings.
void ShowerTransform::ShowerIt()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    while (thePixel != 0)
    {
        thePixel->rx += thePixel->vx;
        thePixel->ry += thePixel->vy;

        //If we go beyond the boundaries of our box (ignoring the ceiling), we reverse the direction of travel.
        if (thePixel->rx+ theImage->X() > endX ||
                thePixel->rx+ theImage->X() < startX) thePixel->vx = 0.6 * -thePixel->vx;
        if (thePixel->ry+ theImage->Y() > endY) thePixel->vy = 0.6 * -thePixel->vy;
        
        //We then affect our next velocity update with our acceleration.
        thePixel->vy += thePixel->ay;
        if (thePixel->vy > 4) thePixel->vy = 4;  //Obviously there is a max to consider.

        thePixel = theImage->GetNextPixel();
    }
}

//This brings all the pixels back to their original position.
void ShowerTransform::SnapIt()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    int Continue = 0;
    while (thePixel != 0)
    {
        if (thePixel->rx == thePixel->ox && thePixel->ry == thePixel->oy)
        {
            //If we're bang-on our original position, we reset the pixel.
            thePixel->vx = 0;
            thePixel->vy = 0;
            thePixel->ay = 0;
        }
        else
        {
            //Else we move along the vector between our current and original position (in ten steps.)
            float movex = (thePixel->ox - thePixel->rx) / 10.0;
            float movey = (thePixel->oy - thePixel->ry) / 10.0;

            thePixel->rx += movex;
            thePixel->ry += movey;

            //If we're close enough, we set it to the original position.
            if (thePixel->rx > thePixel->ox-0.5 && thePixel->rx < thePixel->ox+0.5) thePixel->rx = thePixel->ox;
            if (thePixel->ry > thePixel->oy-0.5 && thePixel->ry < thePixel->oy+0.5) thePixel->ry = thePixel->oy;

            Continue++;
        }
        thePixel = theImage->GetNextPixel();
    }
    if (!Continue)  //If no pixels moved, we're ready to turn off the Transform.
    {
        Active = false;
        Snap = false;
    }
}

//This function turns our Transform on and off.
void ShowerTransform::SwitchTransform()
{
    if (Active)  //If we're turning it off, we set our snap to put pixels back in place.
    {            //The transform will turn itself off after the snap is complete.
        Snap = true;
    }
    else Active = true;
}

void ShowerTransform::UpdateTransform()
{
    if (Snap)
    {
        SnapIt();
    }
    else
    {
        ShowerIt();
    }
}

bool ShowerTransform::IsTransformOn()
{
    return Active;
}
