#include "transform.h"

SizeTransform::SizeTransform()
{
    theImage = 0;
    hMultiplier = 1;
    vMultiplier = 1;
    Active = false;
    Snap = false;
}

//This sets up our transform, giving us a base multiplier of 1.
void SizeTransform::SetTransform(Pixels* Target)
{
    theImage = Target;
    hMultiplier = 1;
    vMultiplier = 1;
}

//When we have to resize our image (based on a new multiplier), this is where we do it.
void SizeTransform::RedoSize()
{
    if (theImage != 0 && Active)
    {
        Pixel* thePixel = theImage->GetFirstPixel();
        while (thePixel != 0)
        {   //We multiply every position by it's appropriate multiplier, thereby moving it to a new position.
            thePixel->rx = thePixel->ox * hMultiplier;
            thePixel->ry = thePixel->oy * vMultiplier;

            thePixel = theImage->GetNextPixel();
        }
    }
}

//This increases the size of the image.
void SizeTransform::UpSize()
{
    hMultiplier += 0.05;
    vMultiplier += 0.05;
    RedoSize();
}

//This decreases the size of the image.
void SizeTransform::DownSize()
{
    hMultiplier -= 0.05;
    vMultiplier -= 0.05;
    if (hMultiplier < 0) hMultiplier = 0;
    if (vMultiplier < 0) vMultiplier = 0;
    RedoSize();
}

//This increases the horizontal size of the image, stretching it.
void SizeTransform::UpHSize()
{
    hMultiplier += 0.05;
    RedoSize();
}

//This shrinks the horizontal size of the image, stretching it.
void SizeTransform::DownHSize()
{
    hMultiplier -= 0.05;
    if (hMultiplier < 0) hMultiplier = 0;
    RedoSize();
}

//This increases the vertical size of the image, stretching it.
void SizeTransform::UpVSize()
{
    vMultiplier += 0.05;
    RedoSize();
}

//This shrinks the vertical size of the image, stretching it.
void SizeTransform::DownVSize()
{
    vMultiplier -= 0.05;
    if (vMultiplier < 0) vMultiplier = 0;
    RedoSize();
}

//Since the Transform doesn't generate a motion, this is left blank.  
void SizeTransform::UpdateTransform()
{

}

void SizeTransform::SwitchTransform()
{
    if (Active)  //When we turn it off, we set each pixel permanently into it's new co-ordinates.
    {
        Pixel* thePixel = theImage->GetFirstPixel();
        while (thePixel != 0)
        {
            thePixel->ox = thePixel->rx;
            thePixel->oy = thePixel->ry;

            thePixel = theImage->GetNextPixel();
        }
        //This sets our end-point to new co-ordinates based on our multipliers.
        theImage->MoveEnd(theImage->EndX() * hMultiplier, theImage->EndY() * vMultiplier);
        hMultiplier = 1;  //We then reset the multipliers for next time they are used.
        vMultiplier = 1;
    }

    Active = !Active;
}

bool SizeTransform::IsTransformOn()
{
    return Active;
}
