#include "transform.h"

//basic constructor
TwistyTransform::TwistyTransform()
{
    theImage = 0;
    moveY = 1;
    moveX = 1;
    Active = false;
    Snap = false;
    Horizontal = false;
}

//This sets up our target image for the move.
void TwistyTransform::SetTransform(Pixels* Target)
{
    theImage = Target;

    Pixel* thePixel = theImage->GetFirstPixel();
    while (thePixel != 0)
    {  //We'll be using the transform co-ordinates as our destination point.  In this case, the destination is the opposite side of the image.
        thePixel->transformx = Target->EndX() - thePixel->ox;  
        thePixel->transformy = Target->EndY() - thePixel->oy;  //If we're gonna flip vertically, we'll need this too.
        thePixel->vx = 1;  //We're just using this to note that the pixel is in motion.
        thePixel = theImage->GetNextPixel();
    }
}

//This function flips our image by moving one pixel at a time to their new destination.
void TwistyTransform::UpdateTransform()
{
    Pixel* thePixel = theImage->GetFirstPixel();
    int loop = 0;
    while (thePixel != 0)
    {
        if (thePixel->vx > 0) loop++;  //If we've got a moving pixel, we gotta keep going.
        if (Horizontal)  //If we're flipping horizontally...
        {
            //The pixel has to be less than moveX to be moved.  moveX represents a line that scans down the image.
            //This gives the image flipping a twisting motion.
            if (thePixel->vx > 0 && thePixel->rx < moveX)
            {
                float movey = thePixel->transformy - thePixel->oy;
                thePixel->ry += movey/70.0;
                if ((movey > 0 && thePixel->ry > thePixel->transformy) ||
                        (movey < 0 && thePixel->ry < thePixel->transformy) ||
                        (movey == 0))
                {
                    thePixel->ry = thePixel->transformy;
                    thePixel->oy = thePixel->ry;
                    thePixel->vx = 0;
                }
            }
        }
        else
        {
            //The pixel has to be less than the moveY line.
            if (thePixel->vx > 0 && thePixel->ry < moveY)
            {
                float movex = thePixel->transformx - thePixel->ox;
                thePixel->rx += movex/70.0;
                if ((movex > 0 && thePixel->rx > thePixel->transformx) ||
                        (movex < 0 && thePixel->rx < thePixel->transformx) ||
                        (movex == 0))
                {
                    thePixel->rx = thePixel->transformx;
                    thePixel->ox = thePixel->rx;
                    thePixel->vx = 0;
                }
            }
        }

        thePixel = theImage->GetNextPixel();
    }

    //The lines that represent the twist have to be moved as a fraction of the total 'size' of the image.
    //This means it will take 30 steps, or we'll have 30 'fragments' that each twist consecutively to produce the effect.
    moveX+= theImage->EndX()/30.0;
    moveY+= theImage->EndY()/30.0;
    if (!loop) //If no pixels are still moving, we turn off our Transform.
    {
        Active = false;
        moveX = 1;
        moveY = 1;
    }
}

//This function turns it on if it isn't on.  The transform shuts itself off when it's done the flip.
void TwistyTransform::SwitchTransform()
{
    if (!Active) Active = true;
}

void TwistyTransform::SetHorizontal()
{
    if (!IsTransformOn())  //It's okay to set horizontal... but not if we're in the middle of something!
        Horizontal = !Horizontal;
}

bool TwistyTransform::IsTransformOn()
{
    return Active;
}
