#include "transform.h"

//basic constructor
WavyTransform::WavyTransform()
{
    theImage = 0;
    startX = 0;
    startY = 0;
    endX = 0;
    endY = 0;
    Active = false;
    Snap = false;
    tick = 0.0;
    Horizontal = true;
    frequency = 0.1;
    amplitude = 3.0;
    speed = 0.3;
}

//This sets the amplitude of the wave.
void WavyTransform::SetAmplitude(float newAmplitude)
{
    amplitude = newAmplitude;
}

//obvious.
void WavyTransform::SetFrequency(float newFrequency)
{
    frequency = newFrequency;
}

//This sets the speed the wave passes by.
void WavyTransform::SetSpeed(float newSpeed)
{
    speed = newSpeed;
}

//This sets the wave to move in the other direction.
//We flick the switch on/off in order to reset the pixels.
void WavyTransform::SetHorizontal()
{
    Horizontal = !Horizontal;
    SwitchTransform();
    SwitchTransform();
}

//This sets us up with our target and the box of the affected area where the wave takes place.
void WavyTransform::SetTransform(Pixels* Target, float sx, float sy, float ex, float ey)
{
    theImage = Target;
    startX = sx;
    startY = sy;
    endX = ex;
    endY = ey;
}

void WavyTransform::UpdateTransform()
{
    tick += speed;
    //The tick above follows the speed set.  Each time tick is increased, the wave moves.
    Pixel* thePixel = theImage->GetFirstPixel();
    while (thePixel != 0)
    {
     float checkx = thePixel->rx + theImage->X();
     float checky = thePixel->ry + theImage->Y();
     if(checkx >= startX && checkx <= endX &&
        checky >= startY && checky <= endY)
     {
        //Move the pixel according to a sin wave.  Basic math made for easy googling.
        if (Horizontal)
        {
            float move = sin((thePixel->oy + tick) * frequency) * amplitude;
            thePixel->rx = thePixel->ox + move;
            thePixel->ry = thePixel->oy;
        }
        else //We move the wave along the y if that's your thing.
        {
            float move = sin((thePixel->ox + tick) * frequency) * amplitude;
            thePixel->ry = thePixel->oy + move;
            thePixel->rx = thePixel->ox;
        }
     }
     else //We reset the pixel if it's left out.  This is to account for motion in/out of the box.
     {
      thePixel->rx = thePixel->ox;
      thePixel->ry = thePixel->oy;    
     }
        thePixel = theImage->GetNextPixel();
    }
}

//This turns the transform on and off.  When it turns off, it resets the image back to it's original position.
void WavyTransform::SwitchTransform()
{
    if (Active)
    {
        Pixel* thePixel = theImage->GetFirstPixel();
        while (thePixel != 0)
        {
            thePixel->rx = thePixel->ox;
            thePixel->ry = thePixel->oy;
            thePixel = theImage->GetNextPixel();
        }
        Active = false;
    }
    else Active = true;
}

bool WavyTransform::IsTransformOn()
{
    return Active;
}
